@@ -5,6 +5,7 @@ use codec::{BinCodec, Codec, Codecs};
 use stub::ServerStub;
 use std::io::Read;
 use std::io::Write;
+use std::marker::PhantomData;
 use std::net::ToSocketAddrs;
 use std::sync::Arc;
 use mco::std::sync::SyncHashMap;
@@ -39,18 +40,50 @@ pub trait Stub {
     fn accept(&self, arg: &[u8], codec: &Codecs) -> Result<Vec<u8>>;
 }
 
-pub trait Handler {
+pub trait Handler: Stub {
     type Req: DeserializeOwned;
     type Resp: Serialize;
+    fn accept(&self, arg: &[u8], codec: &Codecs) -> Result<Vec<u8>> {
+        //.or_else(|e| Result::Err(err!("{}",e)))?
+        let req: Self::Req = codec.decode(arg)?;
+        let data = self.handle(req)?;
+        Ok(codec.encode(data)?)
+    }
     fn handle(&self, req: Self::Req) -> Result<Self::Resp>;
 }
 
 impl<H: Handler> Stub for H {
     fn accept(&self, arg: &[u8], codec: &Codecs) -> Result<Vec<u8>> {
-        //.or_else(|e| Result::Err(err!("{}",e)))?
-        let req: H::Req = codec.decode(arg)?;
-        let data = self.handle(req)?;
-        Ok(codec.encode(data)?)
+        <H as Handler>::accept(self, arg, codec)
+    }
+}
+
+// impl<F> Stub for F where F: Fn(i32)->Result<i32>, F: Sync + Send {
+//     fn accept(&self, arg: &[u8], codec: &Codecs) -> Result<Vec<u8>> {
+//         let req: i32 = codec.decode(arg)?;
+//         let data = self(req)?;
+//         Ok(codec.encode(data)?)
+//     }
+// }
+
+pub struct HandleFn<Req: DeserializeOwned, Resp: Serialize> {
+    pub f: Box<dyn Fn(Req) -> Result<Resp>>,
+}
+
+impl<Req: DeserializeOwned, Resp: Serialize> Handler for HandleFn<Req, Resp> {
+    type Req = Req;
+    type Resp = Resp;
+
+    fn handle(&self, req: Self::Req) -> mco::std::errors::Result<Self::Resp> {
+        (self.f)(req)
+    }
+}
+
+impl<Req: DeserializeOwned, Resp: Serialize> HandleFn<Req, Resp> {
+    pub fn new<F: 'static>(f: F) -> Self where F: Fn(Req) -> Result<Resp> {
+        Self {
+            f: Box::new(f),
+        }
     }
 }
 
@@ -60,6 +93,10 @@ impl Server {
         self.handles.insert(name.to_owned(), Box::new(handle));
     }
 
+    pub fn register_fn<Req: DeserializeOwned + 'static, Resp: Serialize + 'static, F: 'static>(&mut self, name: &str, f: F) where F: Fn(Req) -> Result<Resp> {
+        self.handles.insert(name.to_owned(), Box::new(HandleFn::new(f)));
+    }
+
     pub fn serve<A>(self, addr: A) where A: ToSocketAddrs {
         let listener = TcpListener::bind(addr).unwrap();
         println!(
